<?php

  /*

    All functions and code Copyright Chirag Mehta 2006
    Unless otherwise mentioned
    This code is lazily released under Creative Commons License:
      http://creativecommons.org/licenses/by/2.5/

  */

  require_once(dirname(__FILE__) . "/ignore-large.php");
  require_once(dirname(__FILE__) . "/ignore-small.php");
  require_once(dirname(__FILE__) . "/stem.php");

  class Tagline
  {
    private $taglineID;
    private $tagData      = array(); // columns: header, body (both can be html)
    private $alsoIgnore   = array();
    private $dontIgnore   = array();
    private $combineWords = array();

    // only used for generating tagline
    private $gen_ignore   = array();
    private $gen_manual   = array();
    private $gen_wordfreq = array();
    private $gen_htmltags = array();
    private $gen_tagpeak  = array();
    private $gen_wordstem = array();

    // you can set/change these when using the class
    public $maxTags        = 100;                   // 40 - 200 is a good range for maximum tags to show per cloud
    public $stickiness     = 0.5;                   // (0 = 0% to 1 = 100%) How much effect does past usage of words have on newer clouds

    public $oldColor       = array(50, 100, 100);   // RGB values (0-255)
    public $newColor       = array(255, 255, 255);  // RGB values (0-255)
    public $minFont        = 11;                    // Size in pixels
    public $maxFont        = 48;                    // Size in pixels

    public $ignoreWordList = "large";               // "large" (default) or "small"

    function __construct($taglineID)
    {
      $this->taglineID = $taglineID;
    }

    function loadData($dataSrc)
    {
      if(!is_array($dataSrc))
        $dataSrc = $this->loadTaglineFromXML($dataSrc);

      if(is_array($dataSrc))
        $this->tagData = $dataSrc;
    }

    function loadTaglineFromXML($fl)
    {
      $xml = file_get_contents($fl);

      preg_match_all("'<header>(.*?)</header>'si", $xml, $xmlheader);
      preg_match_all("'<body>(.*?)</body>'si", $xml, $xmlbody);

      if(count($xmlheader[1]) > 0 && count($xmlheader[1]) == count($xmlbody[1]))
      {
        foreach($xmlheader[1] as $key => $val)
        {
          $bodytext = strtolower(trim(strip_tags(str_replace(array("\n", "\r"), " ", $xmlbody[1][$key]))));
          $this->tagData[$key] = array("header" => $val, "body" => $bodytext);
        }
      }
      else
      {
        echo "Error in XML formating. Please verify that the HTML body is escaped.";
      }
    }

    function alsoIgnoreWords($wrd)
    {
      $this->alsoIgnore = array_merge($this->alsoIgnore, $wrd);
    }

    function dontIgnoreWords($wrd)
    {
      $this->dontIgnore = array_merge($this->dontIgnore, $wrd);
    }

    function combineWords($wrd)
    {
      $this->combineWords = array_merge($this->combineWords, $wrd);
    }

    function save()
    {
      if(!defined('TAGLINE_GENPATH'))
        return false;

      $taghtm = $this->generate();
      $tpl_htm = file_get_contents(dirname(__FILE__) . "/template-htm.txt");
      $tpl_htm = str_replace(
                              array("[tagid]", "[taghtm]", "[maxdocs]"),
                              array($this->taglineID, $taghtm, count($this->tagData)),
                              $tpl_htm
                             );

      file_put_contents(TAGLINE_GENPATH . "/" . $this->taglineID . ".htm", $tpl_htm);

      $tpl_phpsrc = file_get_contents(dirname(__FILE__) . "/template-php.txt");
      $tpl_php = str_replace(array("[maxtags]", "[htmfile]"), array($this->maxTags,  "<?php include \"" . $this->taglineID . ".htm\"; ?>"), $tpl_phpsrc);
      file_put_contents(TAGLINE_GENPATH . "/" . $this->taglineID . ".php", $tpl_php);

      $tpl_html = str_replace(array("[maxtags]", "[htmfile]"), array($this->maxTags, $tpl_htm), $tpl_phpsrc);
      file_put_contents(TAGLINE_GENPATH . "/" . $this->taglineID . ".html", $tpl_html);
    }

    function generate()
    {
      if(strtolower(trim($this->ignoreWordList)) == "small")
        $ignore = $GLOBALS["ignore_small"];
      else
        $ignore = $GLOBALS["ignore_large"];

      foreach($ignore as $word)
        $this->gen_ignore[$word] = $word;

      foreach($this->alsoIgnore as $word)
        $this->gen_ignore[$word] = $word;

      foreach($this->dontIgnore as $word)
        if(isset($this->gen_ignore[$word]))
          unset($this->gen_ignore[$word]);

      foreach($this->combineWords as $key => $val)
        $this->gen_manual[strtolower(trim(PorterStemmer::Stem($key)))] = strtolower(trim(PorterStemmer::Stem($val)));

      for($i = 0; $i < count($this->tagData); $i++)
      {
        echo "Processing... Doc# " . ($i + 1) .  "...<br />\n";
        $this->gen_htmltags[$i] = "<div class=tl id=tl_" . $this->taglineID . "_" . ($i + 1) . ">" . $this->getTagCloud($i) ."</div>";
      }

      return implode("\n", $this->gen_htmltags);

    }

    function getTagCloud($docid)
    {
      $ret = "";
      $blogwords = preg_split("/[\s,.;:!\?]+/", $this->tagData[$docid]["body"]);

      if($docid > 0 && count($this->gen_wordfreq) > 0)
      {
        foreach($this->gen_wordfreq as $key => $val)
        {
          $this->gen_wordfreq[$key] *= $this->stickiness;
          if($this->gen_wordfreq[$key] < 0.05)
            unset($this->gen_wordfreq[$key]);
        }
      }

      foreach($blogwords as $word)
      {
        if(strlen($word) > 2 && !ereg('[^a-z]', $word))
        {
          if(isset($this->gen_wordfreq[$word]))
            $this->gen_wordfreq[$word]++;
          else
            $this->gen_wordfreq[$word] = 1;
        }
      }

      $stemfreq = array();
      $stemlead = array();

      if($this->gen_wordfreq)
      {
        foreach($this->gen_wordfreq as $word => $freq)
        {
          if(!isset($this->gen_wordstem[$word]))
            $this->gen_wordstem[$word] = $this->getStem($word);
          $stemfreq[$this->gen_wordstem[$word]][$word] = $freq;
        }

        foreach($stemfreq as $stem => $words)
        {
          $totalfreq = 0;
          $leadword = "";
          $leadfreq = 0;

          foreach($words as $word => $freq)
          {
            $totalfreq += $freq;
            if($freq > $leadfreq)
            {
              $leadword = $word;
              $leadfreq = $freq;
            }
          }
          $stemlead[$leadword] = $totalfreq;
        }

        $this->gen_wordfreq = $stemlead;
      }

      foreach($this->gen_ignore as $val)
        if(isset($this->gen_wordfreq[$val]))
          unset($this->gen_wordfreq[$val]);

      if(count($this->gen_wordfreq) > 0)
      {
        $curwordfreq = $this->gen_wordfreq;
        arsort($curwordfreq, SORT_NUMERIC); // sort by freq
        $curwordfreq = array_slice($curwordfreq, 0, $this->maxTags); // pick top $maxTags
        $freqs = array_values($curwordfreq); // top $maxTags freqs
        ksort($curwordfreq, SORT_STRING); // sort by alpha/key

        $fmax = $freqs[0]; // max freq -> 48px
        $fmin = $freqs[count($freqs)-1]; // min freq -> 12px
        $ftot = 0;

        $peak = array();
        $pmin = 999999;
        $pmax = 0;

        foreach($curwordfreq as $key => $val)
        {
          // freqs for size
          $ftot += $val;

          // peaks for color
          $npeak = $this->getTagPeak($key);
          $peak[$key] = $npeak;
          $pmin = min($pmin, $npeak);
          $pmax = max($pmax, $npeak);
        }

        $prng = max($pmax - $pmin, 1);
        foreach($peak as $key => $val)
          $peak[$key] = ($val - $pmin) / $prng;

        $rethtml = array();
        foreach($curwordfreq as $key => $val)
          $rethtml[] = $this->makeTag($key, $val, $this->getPixelSize($val, $fmax, $fmin, $ftot), $this->getTagPeakColor($peak[$key]));

        $ret = implode("", $rethtml);
      }

      return trim($this->tagData[$docid]["header"]) . "<p>" . $ret . "</p>";
    }

    function getTagPeak($tag)
    {
      if(isset($this->gen_tagpeak[$tag]))
        return $this->gen_tagpeak[$tag];

      $days = 0;
      $peakvar = 0;
      $found = 0;

      foreach($this->tagData as $document)
      {
        $days++;
        if(strpos($document["body"], $tag) !== false)
        {
          $peakvar += $days;
          $found++;
        }
      }

      $this->gen_tagpeak[$tag] = $peakvar / $found;

      return $this->gen_tagpeak[$tag];
    }

    function getTagPeakColor($peak)
    {
      $clr = "";

      foreach($this->oldColor as $rgb => $shade)
        $clr .= substr("00" . dechex($shade + ($peak * ($this->newColor[$rgb] - $shade))), -2);

      return strtoupper($clr);
    }

    function getPixelSize($freq, $fmax, $fmin, $ftot)
    {
      $fontrange = $this->maxFont - $this->minFont;

      // tweak this if all the words seem too similar in size or extremely different
      // rely on the cumulative by x% (0 = 0%, 1 = 100%)
      $cuml_importance = 0.7;

      // sizes based on word's frequency vs total/cumulative frequency
      $sumpx = (($fontrange * $cuml_importance) + 1) * ($fontrange * $cuml_importance) / 2;
      $px = $freq / $ftot * $sumpx;

      // sizes based on word's frequency deviation from max/min frequencies
      $px += pow(($freq-$fmin)/max(1, $fmax-$fmin), 0.8) * ($fontrange * (1 - $cuml_importance));

      return min($this->maxFont, intVal($px + $this->minFont));
    }

    function getStem($word)
    {
      $stem = strtolower(trim(PorterStemmer::Stem($word)));
      if(isset($this->gen_manual[$stem]))
        return $this->gen_manual[$stem];
      return $stem;
    }

    // not valid HTML/XHTML - trying to save precious bytes
    function makeTag($word, $freq, $px, $clr)
    {
      return "<a class=f$px style='color:#$clr'>$word</a> ";
    }

  }

?>
